@@ -99,7 +99,22 @@ get_entry_property(const Entry *entry, char *property)
 	    return xstrdup("object.container.storageFolder");
         /*if (has_entry_detail(entry, DETAIL_URL))
             return xstrdup("object.item.audioItem.audioBroadcast");*/
-	return xstrdup("object.item.audioItem.musicTrack");
+        
+        detail = get_entry_detail(entry, DETAIL_FILE);
+        switch (detail->data.file.item_class) {
+        case ITEM_AUDIO:
+            return xstrdup("object.item.audioItem.musicTrack");
+        case ITEM_IMAGE:
+            return xstrdup("object.item.imageItem.photo");
+        case ITEM_VIDEO:
+            return xstrdup("object.item.videoItem.movie");
+        case ITEM_PLAYLIST:
+        case ITEM_TEXT:
+        case ITEM_UNKNOWN:
+        default:
+            /* FIXME */
+            return NULL;
+        }
     }
 
     if (detail != NULL) {
@@ -321,13 +336,13 @@ add_result_entry(TMap *results, Entry *entry, const char *filter)
     if (has_entry_detail(entry, DETAIL_CHILDREN)) {
         detail = get_entry_detail(entry, DETAIL_CHILDREN);
 	strbuf_appendf(result,
-		"<container id=\"%" PRIi32 "\" parentID=\"%" PRIi32 "\" restricted=\"true\" childCount=\"%" PRIi32 "\">",
+		"<container id=\"%" PRIi32 "\" parentID=\"%" PRIi32 "\" childCount=\"%" PRIi32 "\" restricted=\"1\" searchable=\"1\">",
 		entry->id,
 		entry->parent,
 		detail->data.children.count);
     } else {
 	strbuf_appendf(result,
-		"<item id=\"%" PRIi32 "\" parentID=\"%" PRIi32 "\" restricted=\"true\">",
+		"<item id=\"%" PRIi32 "\" parentID=\"%" PRIi32 "\" restricted=\"1\">",
 		entry->id,
 		entry->parent);
         if (filter_includes_any(filter, "res", "@protocolInfo", "@size", "res@protocolInfo", "res@size", NULL)) {
@@ -340,7 +355,7 @@ add_result_entry(TMap *results, Entry *entry, const char *filter)
 
 		if (S_ISREG(detail->data.file.mode)) { /*if (detail->data.file.size != 0)*/
 	            if (filter_includes_any(filter, "res", "@size", "res@size", NULL))
-                        strbuf_appendf(result, " size=\"%" PRIu32 "\"", detail->data.file.size);
+                        strbuf_appendf(result, " size=\"%zu\"", detail->data.file.size);
 
                     if (has_entry_detail(entry, DETAIL_TAG)) {
                         EntryDetail *tag_detail = get_entry_detail(entry, DETAIL_TAG);
@@ -578,42 +593,42 @@ match_search_expr(SearchExpr *expr, Entry *entry)
         return result;
     case T_EQ:
 	value = get_entry_property(entry, expr->u.binary.property);
-	result = strcmp(value, expr->u.binary.value) == 0;
+	result = value != NULL && strcmp(value, expr->u.binary.value) == 0;
 	free(value);
 	return result;
     case T_NE:
 	value = get_entry_property(entry, expr->u.binary.property);
-	result = strcmp(value, expr->u.binary.value) != 0;
+	result = value != NULL && strcmp(value, expr->u.binary.value) != 0;
 	free(value);
 	return result;
     case T_LT:
 	value = get_entry_property(entry, expr->u.binary.property);
-	result = strcmp(value, expr->u.binary.value) > 0;
+	result = value != NULL && strcmp(value, expr->u.binary.value) > 0;
 	free(value);
 	return result;
     case T_LE:
 	value = get_entry_property(entry, expr->u.binary.property);
-	result = strcmp(value, expr->u.binary.value) >= 0;
+	result = value != NULL && strcmp(value, expr->u.binary.value) >= 0;
 	free(value);
 	return result;
     case T_GT:
 	value = get_entry_property(entry, expr->u.binary.property);
-	result = strcmp(value, expr->u.binary.value) < 0;
+	result = value != NULL && strcmp(value, expr->u.binary.value) < 0;
 	free(value);
 	return result;
     case T_GE:
 	value = get_entry_property(entry, expr->u.binary.property);
-	result = strcmp(value, expr->u.binary.value) <= 0;
+	result = value != NULL && strcmp(value, expr->u.binary.value) <= 0;
 	free(value);
 	return result;
     case T_CONTAINS:
 	value = get_entry_property(entry, expr->u.binary.property);
-	result = strstr(value, expr->u.binary.value) != NULL;
+	result = value != NULL && strstr(value, expr->u.binary.value) != NULL;
 	free(value);
 	return result;
     case T_DOES_NOT_CONTAIN:
 	value = get_entry_property(entry, expr->u.binary.property);
-	result = strstr(value, expr->u.binary.value) == NULL;
+	result = value != NULL && strstr(value, expr->u.binary.value) == NULL;
 	free(value);
 	return result;
     case T_DERIVED_FROM:
@@ -626,12 +641,38 @@ match_search_expr(SearchExpr *expr, Entry *entry)
 	    if (strcmp(expr->u.binary.value, "object.container.storageFolder") == 0)
 		return true;
 	} else {
+	    EntryDetail *detail;
+
 	    if (strcmp(expr->u.binary.value, "object.item") == 0)
 		return true;
-	    if (strcmp(expr->u.binary.value, "object.item.audioItem") == 0)
-		return true;
-	    if (strcmp(expr->u.binary.value, "object.item.audioItem.musicTrack") == 0)
-		return true;
+
+            detail = get_entry_detail(entry, DETAIL_FILE);
+            switch (detail->data.file.item_class) {
+            case ITEM_AUDIO:
+	        if (strcmp(expr->u.binary.value, "object.item.audioItem") == 0)
+		    return true;
+                if (strcmp(expr->u.binary.value, "object.item.audioItem.musicTrack") == 0)
+		    return true;
+                return false;
+            case ITEM_IMAGE:
+	        if (strcmp(expr->u.binary.value, "object.item.imageItem") == 0)
+		    return true;
+                if (strcmp(expr->u.binary.value, "object.item.imageItem.photo") == 0)
+		    return true;
+                return false;
+            case ITEM_VIDEO:
+	        if (strcmp(expr->u.binary.value, "object.item.videoItem") == 0)
+		    return true;
+                if (strcmp(expr->u.binary.value, "object.item.videoItem.movie") == 0)
+		    return true;
+                return false;
+            case ITEM_PLAYLIST:
+            case ITEM_TEXT:
+            case ITEM_UNKNOWN:
+            default:
+                /* FIXME */
+                return false;
+            }            
 	}
 	break;
     }
