@@ -27,7 +27,7 @@
 #include <string.h>		/* C89 */
 #include <stdbool.h>		/* Gnulib, C99; bool */
 #include <sys/types.h>		/* POSIX: time_t */
-#include <upnp.h>		/* libupnp */
+#include <upnp/upnp.h>		/* libupnp */
 #include "hmap.h"
 #include "strbuf.h"
 
@@ -58,6 +58,15 @@ typedef enum {
     DETAIL_URL,		/* mutually exclusive with DETAIL_CHILDREN, FILE */
 } EntryDetailType;
 
+typedef enum {
+    ITEM_IMAGE,		/* refers to imageItem */
+    ITEM_AUDIO,		/* refers to audioItem */
+    ITEM_VIDEO,		/* refers to videoItem */
+    ITEM_PLAYLIST,	/* refers to playlistItem */
+    ITEM_TEXT,		/* refers to textItem */
+    ITEM_UNKNOWN,	/* Not mentioned in standard */
+} ItemClass;
+
 struct _ServiceAction {
     char *name;
     bool (*function)(ActionEvent *);
@@ -100,6 +109,7 @@ struct _EntryURL {
 };
 
 struct _EntryFile {
+    ItemClass item_class;
     const char *mime_type;
     mode_t mode;
     size_t size;
