@@ -19,8 +19,8 @@
  */
 
 #include <config.h>
-#include <upnp.h>		/* libupnp */
-#include <upnptools.h>		/* libupnp */
+#include <upnp/upnp.h>		/* libupnp */
+#include <upnp/upnptools.h>	/* libupnp */
 #include <unistd.h>		/* POSIX */
 #include <fcntl.h>		/* ? */
 #include <getopt.h>		/* Gnulib, GNU */
@@ -46,13 +46,14 @@
 #include "sig2str.h"		/* Gnulib */
 #include "quote.h"		/* Gnulib */
 #include "quotearg.h"		/* Gnulib */
+#include "configmake.h"		/* Gnulib */
 #include "tempfailure.h"
 #include "intutil.h"
 #include "strutil.h"
 #include "gmediaserver.h"
 
 enum {
-    OPT_DISABLE_TAGS,
+    OPT_DISABLE_TAGS = 256,
     OPT_FS_CHARSET,
     OPT_DEVICE_CHARSET,
     OPT_LOG_CHARSET,
@@ -185,6 +186,7 @@ main(int argc, char **argv)
     char *log_charset = NULL;
 
     set_program_name(argv[0]);
+    set_quoting_style(0, escape_quoting_style);
 
     if (setlocale(LC_ALL, "") == NULL)
         warn(_("cannot set locale: %s\n"), errstr);
@@ -302,6 +304,7 @@ main(int argc, char **argv)
             printf(_("      --no-timestamp            do not prepend timestamp to log entries\n"));
             printf(_("      --timestamp[=FORMAT]      prepend timestamp with optional time format\n"));
 	    printf(_("      --profile=NAME            specify profile (see below)\n"));
+	    printf(_("      --file-types=TYPE[,..]    list of file types to serve (see docs)\n"));
             printf(_("      --expire-time=SECONDS     advertisement expire time (default 100)\n"));
             printf(_("      --help                    display this help and exit\n"));
             printf(_("      --version                 output version information and exit\n"));
@@ -370,7 +373,7 @@ main(int argc, char **argv)
         if (fs_to_utf8 == (iconv_t) -1)
             die(_("cannot create character set convertor from %s to %s\n"), quotearg(fs_charset), "UTF-8");
     }
-    say(4, "Using local file system character set %s\n", quote(fs_charset == NULL ? "UTF-8" : fs_charset));
+    say(4, _("Using local file system character set %s\n"), quote(fs_charset == NULL ? "UTF-8" : fs_charset));
 
     if (device_charset != NULL) {
         utf8_to_device = iconv_open(device_charset, "UTF-8");
