@@ -38,6 +38,8 @@
 #include <ctype.h>		/* C89 */
 #include <inttypes.h>		/* ? */
 #include <iconv.h>		/* POSIX */
+#include <magic.h>		/* libmagic */
+#include <upnp/ithread.h>	/* libupnp */
 #include "iconvme.h"		/* Gnulib */
 #include "gettext.h"		/* Gnulib/gettext */
 #define _(s) gettext(s)
@@ -75,6 +77,7 @@ typedef enum {
     FILE_PLS,
     FILE_M3U,
     FILE_EXTM3U,
+    FILE_JPG,
     FILE_UNKNOWN,
     FILE_TYPES_COUNT,
 } FileType;
@@ -91,11 +94,12 @@ static const char *file_type_mime_types[] = {
     [FILE_MP3_ID3]	= "audio/mpeg",
     [FILE_WMA] 		= "audio/x-ms-wma",
     [FILE_RIFF_WAVE] 	= "audio/x-wav",
-    [FILE_M4A] 		= "audio/mp4",  
+    [FILE_M4A] 		= "audio/mp4",
     [FILE_OGG]		= "audio/vorbis",
     [FILE_PLS] 		= "audio/x-scpls",
     [FILE_M3U] 		= "audio/m3u",
     [FILE_EXTM3U] 	= "audio/m3u",
+    [FILE_JPG]		= "image/jpeg",
     [FILE_UNKNOWN] 	= "application/octet-stream",
 };
 
@@ -109,9 +113,11 @@ static const char *file_type_names[] = {
     [FILE_PLS] 		= "pls",
     [FILE_M3U] 		= "m3u",
     [FILE_EXTM3U] 	= "m3u", /* possibly extm3u in the future */
+    [FILE_JPG]		= "jpg",
     [FILE_UNKNOWN] 	= "unknown",
 };
 
+/* FIXME: handle translation for these? */
 static const char *file_type_descs[] = {
     [FILE_MP3] 		= "MPEG-1 Audio Layer 3 (MP3)",
     [FILE_MP3_ID3]	= "MPEG-1 Audio Layer 3 (MP3) with leading ID3 tag",
@@ -122,6 +128,20 @@ static const char *file_type_descs[] = {
     [FILE_PLS]		= "PLS playlist",
     [FILE_M3U]		= "Simple M3U playlist",
     [FILE_EXTM3U]	= "Extended M3U playlist",
+    [FILE_JPG]		= "JPEG image"
+};
+
+static ItemClass file_type_item_classes[] = {
+    [FILE_MP3]         = ITEM_AUDIO,
+    [FILE_MP3_ID3]     = ITEM_AUDIO,
+    [FILE_WMA]         = ITEM_AUDIO,
+    [FILE_RIFF_WAVE]   = ITEM_AUDIO,
+    [FILE_M4A]         = ITEM_AUDIO,
+    [FILE_OGG]         = ITEM_AUDIO,
+    [FILE_PLS]         = ITEM_PLAYLIST,
+    [FILE_M3U]         = ITEM_PLAYLIST,
+    [FILE_EXTM3U]      = ITEM_PLAYLIST,
+    [FILE_JPG]         = ITEM_IMAGE,
 };
 
 static Entry *scan_entry(const char *fullpath, const char *name, int32_t parent, int indent_size, InodeList *inl);
@@ -135,6 +155,7 @@ static Entry **entries;
 static iconv_t iso8859_1_to_utf8;
 static iconv_t utf16_to_utf8;
 #endif
+static magic_t magic_cookie;
 char *file_types = DEFAULT_FILE_TYPES;
 static ithread_mutex_t metadata_mutex;
 bool tags_enabled = true;
@@ -233,75 +254,54 @@ attempt_read_at(int fd, size_t size, off_t pos, uint8_t *buf, const char *fullpa
 static FileType
 check_file_content_type(const char *fullpath)
 {
+    const char *magic;
     int fd;
     uint8_t buf[11];
+    int c;
+
+    magic = magic_file(magic_cookie, fullpath);
+    if (magic == NULL) {
+	warn(_("%s: cannot identify file type: %s\n"), quotearg(conv_filename(fullpath)), magic_error(magic_cookie));
+	return FILE_UNKNOWN;
+    }
+
+    if (strcmp(magic, "application/octet-stream") != 0
+	    && strncmp(magic, "text/plain", 10) != 0) {
+	struct {
+	    FileType id;
+	    char *mime;
+	} mime_map[] = {
+	    { FILE_MP3, 	"audio/mpeg" }, /* XXX: FILE_MP3_ID3? */
+	    { FILE_WMA, 	"audio/x-ms-wma" },
+	    { FILE_RIFF_WAVE,	"audio/x-wav" },
+	    { FILE_M4A,		"video/mp4" },
+	    { FILE_M4A,		"audio/mp4" },
+	    { FILE_OGG,		"application/ogg" },
+	    { FILE_JPG,		"image/jpeg" },
+            { 0, },
+        };
+        for (c = 0; mime_map[c].mime != NULL; c++) {
+            if (strcmp(magic, mime_map[c].mime) == 0)
+                return mime_map[c].id;
+        }
+    }
 
-    /* XXX: Use stdio instead of fd syscalls */
     fd = open(fullpath, O_RDONLY);
     if (fd < 0) {
-	warn(_("%s: cannot open for reading: %s\n"), quotearg(conv_filename(fullpath)), errstr);
-	return FILE_UNKNOWN;
+        warn(_("%s: cannot open for reading: %s\n"), quotearg(conv_filename(fullpath)), errstr);
+        return FILE_UNKNOWN;
     }
-
-    if (!attempt_read_at(fd, 4, 0, buf, fullpath)) {
+    if (!attempt_read_at(fd, 11, 0, buf, fullpath)) {
         close(fd);
-	return FILE_UNKNOWN;
+        return FILE_UNKNOWN;
     }
 
-    /* MPEG ADTS, layer III, v1 */
-    if (buf[0] == 0xFF && (buf[1] & 0xFE) == 0xFA) {
-	close(fd); /* Ignore errors since we opened for reading */
-	return FILE_MP3;
-    }
-    /* MP3 file with ID3 tag */
-    if (buf[0] == 'I' && buf[1] == 'D' && buf[2] == '3') {
-	close(fd); /* Ignore errors since we opened for reading */
-	return FILE_MP3_ID3;
-    }
     /* Microsoft ASF */
     if (buf[0] == 0x30 && buf[1] == 0x26 && buf[2] == 0xb2 && buf[3] == 0x75) {
-	close(fd); /* Ignore errors since we opened for reading */
-	return FILE_WMA;
-    }
-    /* RIFF (little-endian), WAVE audio */
-    if (buf[0] == 'R' && buf[1] == 'I' && buf[2] == 'F' && buf[3] == 'F') {
-	if (!attempt_read_at(fd, 4, 8, buf, fullpath)) {
-	    close(fd);
-	    return FILE_UNKNOWN;
-        }
-	if (buf[0] == 'W' && buf[1] == 'A' && buf[2] == 'V' && buf[3] == 'E') {
-	    close(fd); /* Ignore errors since we opened for reading */
-	    return FILE_RIFF_WAVE;
-	}
+        close(fd); /* Ignore errors since we opened for reading */
+        return FILE_WMA;
     }
 
-    /* Ogg Vorbis audio */
-    if (buf[0] == 'O' && buf[1] == 'g' && buf[2] == 'g' && buf[3] == 'S') {
-        if (!attempt_read_at(fd, 7, 28, buf+4, fullpath)) {
-            close(fd);
-            return FILE_UNKNOWN;
-        }
-        if (buf[4]==1 && buf[5]=='v' && buf[6]=='o' && buf[7]=='r' && buf[8]=='b' && buf[9]=='i' && buf[10]=='s') {
-	    close(fd); /* Ignore errors since we opened for reading */
-	    return FILE_OGG;
-        }
-    }
-
-    /* ISO Media, MPEG v4 system, iTunes AAC-LC */
-    if (!attempt_read_at(fd, 4, 4, buf+4, fullpath)) {
-        close(fd);
-	return FILE_UNKNOWN;
-    }
-    if (buf[4] == 'f' && buf[5] == 't' && buf[6] == 'y' && buf[7] == 'p') {
-	if (!attempt_read_at(fd, 4, 8, buf+8, fullpath)) {
-	    close(fd);
-	    return FILE_UNKNOWN;
-        }
-	if (buf[8] == 'M' && buf[9] == '4' && buf[10] == 'A') {
-	    close(fd); /* Ignore errors since we opened for reading */
-	    return FILE_M4A;
-	}
-    }
 
     /* Extended M3U */
     if (buf[0]=='#' && buf[1]=='E' && buf[2]=='X' && buf[3]=='T' && buf[4]=='M' && buf[5]=='3' && buf[6]=='U' && (buf[7]=='\r' || buf[7]=='\n')) {
@@ -309,19 +309,14 @@ check_file_content_type(const char *fullpath)
         return FILE_EXTM3U;
     }
     /* Playlist (PLS) */
-    if (!attempt_read_at(fd, 4, 8, buf+8, fullpath)) {
-        close(fd);
-	return FILE_UNKNOWN;
-    }
     if (memcmp(buf, "[playlist]", 10) == 0 && (buf[10] == '\r' || buf[10] == '\n')) {
-	close(fd); /* Ignore errors since we opened for reading */
-	return FILE_PLS;
+        close(fd); /* Ignore errors since we opened for reading */
+        return FILE_PLS;
     }
-
     /* Simple M3U */
     if (ends_with_nocase(fullpath, ".m3u")) {
-	close(fd); /* Ignore errors since we opened for reading */
-	return FILE_M3U;
+        close(fd); /* Ignore errors since we opened for reading */
+        return FILE_M3U;
     }
 
     close(fd); /* Ignore errors since we opened for reading */
@@ -800,6 +795,7 @@ scan_entry(const char *fullpath, const char *name, int32_t parent, int indent_si
         detail->data.file.mtime = sb.st_mtime;
         detail->data.file.filename = xstrdup(fullpath);
 	detail->data.file.mime_type = file_type_mime_types[type];
+	detail->data.file.item_class = file_type_item_classes[type];
 	/* XXX: dump size, mtime, ?? */
 
 	if (tags_enabled && (type == FILE_MP3 || type == FILE_MP3_ID3)) {
@@ -915,6 +911,7 @@ scan_entry(const char *fullpath, const char *name, int32_t parent, int indent_si
         detail->data.file.mtime = sb.st_mtime;
         detail->data.file.filename = xstrdup(fullpath);
 	detail->data.file.mime_type = file_type_mime_types[FILE_MP3]; /* FIXME: unknown type. */
+	detail->data.file.item_class = file_type_item_classes[FILE_MP3];
 
 	free(indent);
 	return entry;
@@ -928,11 +925,25 @@ scan_entry(const char *fullpath, const char *name, int32_t parent, int indent_si
 bool
 init_metadata(void)
 {
+    magic_cookie = magic_open(MAGIC_SYMLINK|MAGIC_MIME|MAGIC_ERROR);
+    if (magic_cookie == NULL) {
+        warn(_("cannot initialize magic library\n"));
+        return false;
+    }
+
+    say(4, _("Loading default magic database\n"));
+    if (magic_load(magic_cookie, NULL) != 0) {
+        warn(_("cannot load magic database - %s\n"), magic_error(magic_cookie));
+        magic_close(magic_cookie);
+        return false;
+    }
+
 #ifdef HAVE_ID3LIB
     if (tags_enabled) {
         iso8859_1_to_utf8 = iconv_open("UTF-8", "ISO8859-1");
         if (iso8859_1_to_utf8 == (iconv_t) -1) {
              warn(_("cannot create character set convertor from %s to %s\n"), "ISO8859-1", "UTF-8");
+             magic_close(magic_cookie);
              return false;
         }
         /* ID3 <= v2.3 uses UTF-16, because the BOM is included. Or to be more correct,
@@ -942,6 +953,8 @@ init_metadata(void)
         utf16_to_utf8 = iconv_open("UTF-8", "UTF-16");
         if (utf16_to_utf8 == (iconv_t) -1) {
              warn(_("cannot create character set convertor from %s to %s\n"), "UTF-16", "UTF-8");
+             iconv_close(iso8859_1_to_utf8); /* ignore errors */
+             magic_close(magic_cookie);
              return false;
         }
 	id3 = ID3Tag_New();
@@ -963,8 +976,8 @@ scan_entries(char **pathv, int pathc, int indent_size)
 	int32_t *children;
 	EntryDetail *detail;
 	uint32_t c;
+	uint32_t child_count = 0;
 
-	root_entry = make_entry(ROOT_ENTRY_NAME, -1, true);
 	children = xmalloc(sizeof(int32_t) * pathc);
 	for (c = 0; c < pathc; c++) {
 	    Entry *entry;
@@ -983,12 +996,19 @@ scan_entries(char **pathv, int pathc, int indent_size)
             }
 
 	    entry = scan_entry(pathv[c], name, -1, indent_size, NULL);
-	    children[c] = entry->id;
+	    if (entry != NULL)
+	        children[child_count++] = entry->id;
 	    free(name);
 	}
-	detail = get_entry_detail(root_entry, DETAIL_CHILDREN);
-	detail->data.children.count = pathc;
-	detail->data.children.list = children;
+	if (child_count != 0) {
+	    root_entry = make_entry(ROOT_ENTRY_NAME, -1, true);
+	    detail = get_entry_detail(root_entry, DETAIL_CHILDREN);
+	    detail->data.children.count = child_count;
+	    detail->data.children.list = children;
+        } else {
+            root_entry = NULL;
+            free(children);
+        }
     } else {
 	root_entry = scan_entry(pathv[0], ROOT_ENTRY_NAME, -1, indent_size, NULL);
     }
