@@ -1,6 +1,6 @@
 /* strbuf.c - The string buffer data-structure.
  *
- * Copyright (C) 2004-2005 Oskar Liljeblad
+ * Copyright (C) 2004, 2005 Oskar Liljeblad
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,12 @@
  *
  */
 
+/* Depends on
+ * gl_MODULES([strnlen vasprintf xalloc minmax])
+ * Optionally
+ * gl_MODULES([stdint])
+ */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -25,6 +31,7 @@
 #include <stdarg.h>		/* C89 */
 #include <stdlib.h>		/* C89 */
 #include <string.h>		/* C89 */
+#include "strnlen.h"		/* Gnulib */
 #include "vasprintf.h"    	/* Gnulib */
 #include "xalloc.h"             /* Gnulib */
 #include "minmax.h"             /* Gnulib */
@@ -307,6 +314,7 @@ strbuf_set_length(StrBuf *sb, uint32_t new_length)
     sb->len = new_length;
 }
 
+/* XXX: the terminating null-byte counts as 1 in min_capacity */
 void
 strbuf_ensure_capacity(StrBuf *sb, uint32_t min_capacity)
 {
@@ -340,3 +348,34 @@ strbuf_char_at(StrBuf *sb, int32_t sp)
 {
     return sb->buf[normalize_strbuf_pos(sb, sp)];
 }
+
+#if 0
+char
+strbuf_set_char_at(StrBuf *sb, int32_t sp, char chr)
+{
+    char old;
+
+    sp = normalize_strbuf_pos(sb, sp);
+    old = str[sp];
+    str[sp] = chr;
+    if (sp == sb->len) {
+        sb->len++;
+        strbuf_ensure_capacity(sb, sb->len);
+        str[sb->len] = '\0'
+    }
+    return old;
+}
+
+void
+strbuf_replace_strbuf(StrBuf *sb, int32_t sp, int32_t ep, StrBuf *strbuf)
+{
+    strbuf_replace_data_n(sb,sp,ep,1,strbuf->buf,strbuf->len);
+}
+
+char
+strbuf_delete_char(StrBuf *sb, int32_t sp)
+{
+
+}
+
+#endif
