@@ -20,8 +20,8 @@
 
 #include <config.h>
 #include <stdbool.h>		/* Gnulib, C99 */
-#include <upnp.h>		/* libupnp */
-#include <upnptools.h>		/* libupnp */
+#include <upnp/upnp.h>		/* libupnp */
+#include <upnp/upnptools.h>	/* libupnp */
 #include <assert.h>		/* C89 */
 #include <netinet/in.h>		/* ?; inet_ntoa */
 #include <arpa/inet.h>		/* ?; inet_ntoa */
