@@ -19,14 +19,80 @@
  */
 
 #include <config.h>
-#include <stdint.h>
-#include <string.h>
+#include <stdbool.h>		/* Gnulib, POSIX, C99 */
+#include <stdint.h>		/* Gnulib, POSIX, C99 */
+#include <string.h>		/* C89 */
+#include <stdlib.h>		/* C89 */
 #include <ctype.h>
-#include "xalloc.h"
+#include "xalloc.h"		/* Gnulib */
+#include "xstrndup.h"		/* Gnulib */
+#include "gmediaserver.h"
+#include "intutil.h"
 
 #define HEXCHRLC(x) ((x) >= 10 ? 'a'+(x) : '0'+(x))
 #define HEXCHRUC(x) ((x) >= 10 ? 'A'+(x) : '0'+(x))
 
+void
+free_url(URL *url)
+{
+    free(url->hostname);
+    free(url->path);
+    free(url);
+}
+
+/* Parse an http URL that does not have logon information (user and
+ * possibly password).
+ * This function really needs to be rewritten. Perhaps we can create 
+ * a lexer to this?
+ */
+URL *
+parse_simple_url(const char *urlstr)
+{
+    char *next;
+    char *next2;
+    char *hostname;
+    uint16_t port;
+    char *path;
+    URL *url;
+
+    if (strncasecmp(urlstr, "http://", 7) != 0)
+        return NULL;
+
+    next = strpbrk(urlstr+7, ":/");
+    if (next == NULL) {
+        hostname = xstrdup(urlstr+7);
+        port = 80;
+        path = xstrdup("/");
+    } else if (*next == ':') {
+        next2 = strchr(next+1, '/');
+        if (next2 == NULL) {
+            if (!parse_uint16(next+1, &port))
+                return NULL;
+            path = xstrdup("/");
+        } else {
+            char *tmp = xstrndup(next+1, next2-next-1);
+            if (!parse_uint16(tmp, &port)) { /* XXX: should make sure tmp[0]=='\0' is not accepted */
+                free(tmp);
+                return NULL;
+            }
+            free(tmp);
+            path = xstrdup(next2);
+        }
+        hostname = xstrndup(urlstr+7, next-urlstr-7);
+    } else {
+        port = 80;
+        hostname = xstrndup(urlstr+7, next-urlstr-7);
+        path = xstrdup(next);
+    }
+    
+    url = xmalloc(sizeof(URL));
+    url->port = port;
+    url->hostname = hostname;
+    url->path = path;
+
+    return url;
+}
+
 void
 sgmlescape(const char *str, char *target, uint32_t *length)
 {
