@@ -23,6 +23,9 @@
 #include <netinet/in.h>		/* POSIX */
 #include <netdb.h>		/* POSIX */
 #include <inttypes.h>		/* POSIX */
+#include <unistd.h>		/* POSIX */
+#include <sys/time.h>		/* ? */
+#include <time.h>		/* POSIX */
 #include <stdio.h>		/* C89 */
 #include "gettext.h"            /* Gnulib/gettext */
 #define _(s) gettext(s)
