@@ -274,7 +274,7 @@ webserver_read(UpnpWebFileHandle fh, char *buf, size_t buflen)
     WebServerFile *file = (WebServerFile *) fh;
     ssize_t len = -1;
 
-    say(4, _("Attempting to read %d bytes at %d from %s\n"), buflen, file->pos, quote(conv_filename(file->fullpath)));
+    say(4, _("Attempting to read %zu bytes at %zu from %s\n"), buflen, file->pos, quote(conv_filename(file->fullpath)));
 
     switch (file->type) {
     case FILE_LOCAL:
@@ -282,7 +282,7 @@ webserver_read(UpnpWebFileHandle fh, char *buf, size_t buflen)
 	len = read(file->detail.local.fd, buf, buflen);
 	break;
     case FILE_MEMORY:
-	len = min(buflen, file->detail.memory.len - file->pos);
+	len = MIN(buflen, file->detail.memory.len - file->pos);
 	memcpy(buf, file->detail.memory.contents + file->pos, len);
 	break;
     case FILE_URL:
@@ -316,15 +316,15 @@ webserver_seek(UpnpWebFileHandle fh, long offset, int origin)
 
     switch (origin) {
     case SEEK_SET:
-	say(4, _("Attempting to seek to %ld (was at %d) in %s\n"), offset, file->pos, quote(conv_filename(file->fullpath))); /* XXX: use quote or quotearg here? */
+	say(4, _("Attempting to seek to %ld (was at %zu) in %s\n"), offset, file->pos, quote(conv_filename(file->fullpath))); /* XXX: use quote or quotearg here? */
 	newpos = offset;
 	break;
     case SEEK_CUR:
-	say(4, _("Attempting to seek by %ld from %d in %s\n"), offset, file->pos, quote(conv_filename(file->fullpath))); /* XXX: use quote or quotearg here? */
+	say(4, _("Attempting to seek by %ld from %zu in %s\n"), offset, file->pos, quote(conv_filename(file->fullpath))); /* XXX: use quote or quotearg here? */
 	newpos = file->pos + offset;
 	break;
     case SEEK_END:
-	say(4, _("Attempting to seek by %ld from end (was at %d) in %s\n"), offset, file->pos, quote(conv_filename(file->fullpath))); /* XXX: use quote or quotearg here? */
+	say(4, _("Attempting to seek by %ld from end (was at %zu) in %s\n"), offset, file->pos, quote(conv_filename(file->fullpath))); /* XXX: use quote or quotearg here? */
 	if (file->type == FILE_LOCAL) {
 	    struct stat sb;
 	    if (stat(file->fullpath, &sb) < 0) {
